<?php
/**
* @package RSComments!
* @copyright (C) 2014 www.rsjoomla.com
* @license GPL, http://www.gnu.org/licenses/gpl-2.0.html
*/

defined('_JEXEC') or die('Restricted access');

abstract class RSCommentsHelper {
	
	// Get component configuration
	public static function getConfig($name = null) {
		static $config;
		
		if (!is_object($config)) {
			$db		= JFactory::getDbo();
			$query	= $db->getQuery(true);
			$config = new stdClass();
			
			$query->clear();
			$query->select($db->qn('params'));
			$query->from($db->qn('#__extensions'));
			$query->where($db->qn('type').' = '.$db->q('component'));
			$query->where($db->qn('element').' = '.$db->q('com_rscomments'));
			$db->setQuery($query);
			$params = $db->loadResult();
			
			// Convert the params to an object.
			if (is_string($params)) {
				$temp = new JRegistry;
				$temp->loadString($params);
				$config = $temp->toObject();
			}
		}
		
		if ($name != null) {
			if (isset($config->{$name})) return $config->{$name};
				else return false;
		} else {
			return $config;
		}
	}
	
	// Check for Joomla! 3.0
	public static function isJ3() {
		return version_compare(JVERSION, '3.0', '>=');
	}

	// Get update code
	public static function genKeyCode() {
		if ($code = RSCommentsHelper::getConfig('global_register_code')) {
			$version = new RSCommentsVersion();
			return md5($code.$version->key);
		}
	}
	
	//set scripts
	public static function setScripts($client = 'administrator') {
		$doc = JFactory::getDocument();

		JHTML::_('behavior.framework',true);
		
		if ($client == 'administrator') {
			if (RSCommentsHelper::isJ3()) {
				$doc->addStyleSheet(JURI::root(true).'/administrator/components/com_rscomments/assets/css/j3.css?v='._RSCOMMENTS_VERSION);
				JHtml::_('formbehavior.chosen', 'select');
			} else {
				$doc->addStyleSheet(JURI::root(true).'/administrator/components/com_rscomments/assets/css/j2.css?v='._RSCOMMENTS_VERSION);
			}
			
			$doc->addStyleSheet(JURI::root(true).'/administrator/components/com_rscomments/assets/css/style.css?v='._RSCOMMENTS_VERSION);
		} else  {
			$doc->addStyleSheet(JURI::root(true).'/components/com_rscomments/assets/css/style.css?v='._RSCOMMENTS_VERSION);
			$doc->addScript(JURI::root(true).'/components/com_rscomments/assets/js/rscomments.js?v='._RSCOMMENTS_VERSION);
		}
	}
	
	// Clean the cache
	public static function cleancache() {
		JFactory::getCache('com_rscomments')->clean();
	}
	
	public static function showDate($date, $format = null) {
		$date_format = !is_null($format) ? $format : RSCommentsHelper::getConfig('date_format');
		return JHTML::date($date, $date_format);
	}

	public static function cleanComment($comment) {
		$patterns = array();
		$replacements = array();
		
		$patterns[] = '/\[b\](.*?)\[\/b\]/i';
		$replacements[] = '\\1';

		$patterns[] = '/\[i\](.*?)\[\/i\]/i';
		$replacements[] = '\\1';

		$patterns[] = '/\[u\](.*?)\[\/u\]/i';
		$replacements[] = '\\1';

		$patterns[] = '/\[s\](.*?)\[\/s\]/i';
		$replacements[] = '\\1';

		$patterns[] = '/\[url\]([ a-zA-Z0-9\:\/\-\?\&\.\=\_\~\#\']*)\[\/url\]/i';
		$replacements[] = '\\1';

		$patterns[] = '#\[img\](http:\/\/)?([^\s\<\>\(\)\"\']*?)\[\/img\]#i';
		$replacements[] = '\\2';

		$patterns[] = '#\[code\](.*?)\[\/code\]#ism';
		$replacements[] = '\\1';

		//YOUTUBE
		$patterns[] = '/\[youtube\](.+?)\[\/youtube\]/';
		$replacements[] = '\\1';
		
		//GOOGLE VIDEOS
		$patterns[] = '/\[google\](.+?)\[\/google\]/';
		$replacements[] = '\\1';
			
		$comment = preg_replace($patterns, $replacements, $comment);
		
		
		// QUOTE
		$quotePattern = '#\[quote\s?name=\"([^\"\'\<\>\(\)]+)+\"\](<br\s?\/?\>)*(.*?)(<br\s?\/?\>)*\[\/quote\]#i';
		$quoteReplace = '\\3';
		while(preg_match($quotePattern, $comment)) {
			$comment = preg_replace($quotePattern, $quoteReplace, $comment);
		}
		$quotePattern = '#\[quote[^\]]*?\](<br\s?\/?\>)*([^\[]+)(<br\s?\/?\>)*\[\/quote\]#i';
		$quoteReplace = '\\2';
		while(preg_match($quotePattern, $comment)) {
			$comment = preg_replace($quotePattern, $quoteReplace, $comment);
		}

		$comment = preg_replace('#\[\/?(b|i|u|s|url|img|list|quote|code)\]#', '', $comment);
		$smiley = array(':confused:',':cool:',':cry:',':laugh:',':lol:',':normal:',':blush:',':rolleyes:',':sad:',':shocked:',':sick:',':sleeping:',':smile:',':surprised:',':tongue:',':unsure:',':whistle:',':wink:');
		$comment = str_replace($smiley,'',$comment);
		
		return $comment;
	}
	
	public static function loadLang($admin = false) {
		$lang = JFactory::getLanguage();
		$from = $admin ? JPATH_ADMINISTRATOR : JPATH_SITE;
		
		$lang->load('com_rscomments', $from, 'en-GB', true);
		$lang->load('com_rscomments', $from, $lang->getDefault(), true);
		$lang->load('com_rscomments', $from, null, true);
		
		return true;
	}
	
	public static function ArticleTitle($component, $component_id) {
		$components = array('com_content', 'com_rsblog', 'com_k2', 'com_flexicontent');
		
		if(!empty($component_id) && in_array($component,$components)) {
			$db		= JFactory::getDBO();
			$query	= $db->getQuery(true);

			$query->select($db->qn('title'));
			switch($component) {
				case 'com_content':
					$query->from($db->qn('#__content'));
				break;
				case 'com_rsblog':
					$query->from($db->qn('#__rsblog_posts'));
				break;
				case 'com_k2':
					$query->from($db->qn('#__k2item'));
				break;
				case 'com_flexicontent':
					$query->from($db->qn('#__flexicontent_items'));
				break;
			}

			$query->where($db->qn('id').' = '.$component_id);
			$db->setQuery($query);

			return $db->loadResult();
		} else 
			return '';
	}
	
	public static function component($option) {
		$lang = JFactory::getLanguage();
		$lang->load($option.'.sys');
		$component = strtoupper($option);
		
		return $lang->hasKey($component) ? JText::_($component) : $option;
	}
	
	public static function getComponents() {
		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true);
		$values	= array();
		
		$query->select($db->qn('element'))
			->from($db->qn('#__extensions'))
			->where($db->qn('type').' = '.$db->q('component'))
			->where($db->qn('enabled').' = 1')
			->where("`element` NOT IN ('com_admin', 'com_wrapper', 'com_weblinks', 'com_users', 'com_user', 'com_templates', 'com_search','com_poll', 'com_plugins', 'com_newsfeeds', 'com_modules', 'com_messages', 'com_menus', 'com_media', 'com_massmail', 'com_mailto', 'com_languages', 'com_installer', 'com_cpanel', 'com_config', 'com_cache', 'com_checkin', 'com_finder', 'com_joomlaupdate', 'com_login', 'com_redirect', 'com_ajax', 'com_contenthistory', 'com_postinstall', 'com_tags')")
			->order('ordering, name');
		$db->setQuery($query);
		if ($extensions = $db->loadColumn()) {
			$values = array_merge($values, $extensions);
		}
		
		$query->clear()
			->select('DISTINCT('.$db->qn('option').')')
			->from($db->qn('#__rscomments_comments'));
		$db->setQuery($query);
		if ($options = $db->loadColumn()) {
			$values = array_merge($values,$options);
		}
		
		$values = array_unique($values);
		return $values;
	}
	
	public static function language($tag) {
		$lang = JFactory::getLanguage();
		$languages = $lang->getKnownLanguages();
		
		if ($languages && $languages[$tag]) {
			return !empty($languages[$tag]['name']) ? $languages[$tag]['name'] : $tag;
		}
		
		return $tag;
	}
	
	public static function canAdd() {
		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true);
		
		$query->clear();
		$query->select('a.id');
		$query->from($db->qn('#__usergroups') . ' AS a');
		$query->join('LEFT', $db->qn('#__usergroups') . ' AS b ON a.lft > b.lft AND a.rgt < b.rgt');
		$query->group('a.id, a.title, a.lft, a.rgt');
		$query->order('a.lft ASC');
		$db->setQuery($query);
		
		if ($groups = $db->loadColumn()) {
			JArrayHelper::toInteger($groups);
			
			$query->clear();
			$query->select('gid');
			$query->from($db->qn('#__rscomments_groups'));
			$db->setQuery($query);
			if ($gids = $db->loadColumn()) {
				JArrayHelper::toInteger($gids);
				$diff = array_diff($groups,$gids);
				
				if (empty($diff)) {
					return false;
				}
			}
		}
		
		return true;
	}
}